//====================================================================================================
//
//   NOMBRE DEL ARCHIVO:	Imagen.h  VERSIN 2.2
//
//   DESCRIPCION:	Define la clase CImagen
//
//   AUTOR:		Francisco Javier Gmez Quesada
//
//	 FECHA:		23/Febrero/1999
//
//   ULTIMA REVISION:	Noviembre 2004
//
//====================================================================================================

#if !defined(AFX_OBJETOIMAGEN_H__9E6BA040_AB96_11D2_9260_00C0DF46F28C__INCLUDED_)
#define AFX_OBJETOIMAGEN_H__9E6BA040_AB96_11D2_9260_00C0DF46F28C__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000
#include <math.h>	// Para Fourier (usa struct _complex)

class AFX_EXT_CLASS CImagen 
{

#define VERTICAL	0
#define HORIZONTAL	1

#define SUMA		    0		// Operadores Aritmticos
#define DIFERENCIA	    1
#define PRODUCTO	    2
#define VALOR_MEDIO	    3
#define EL_MAS_CLARO	4
#define EL_MAS_OBSCURO	5

#define NOT	    0		// Operadores Lgicos
#define AND	    1
#define OR	    2
#define XOR	    3
#define NAND	4
#define NOR		5

#define ALEATORIO	0			// Tipos de Ruido
#define UNIFORME	1
#define BLANCO		0			
#define COLOREADO	1

#define PUNTOSAISLADOS	 0		// Filtros
#define LVERTICAL		 1
#define LHORIZONTAL		 2
#define LDIAGONAL1		 3
#define LDIAGONAL2		 4
#define	REALCE1			 5
#define	REALCE2			 6
#define	REALCE3			 7
#define	REALCE4			 8
#define	REALCE5			 9
#define	REALCE6			10
#define	REALCE7			11
#define	REALCE8			12
#define	REALCE9			13
#define	REALCE10		14
#define	REALCE11		15
#define MEDIA3X3		16
#define MEDIA5X5		17
#define MEDIANA3X3		18
#define MEDIANA5X5		19
#define	GAUSS1  		20
#define	GAUSS2  		21
#define	GAUSS3  		22
#define	SUAVIZADO1		23
#define	SUAVIZADO2		24
#define	SUAVIZADO3		25
#define EMBORRONAR		26
#define ROBERTS			27
#define ROBERTSV		28
#define ROBERTSH		29
#define SOBEL			30
#define	SOBELV			31
#define SOBELH			32
#define PREWITT			33
#define PREWITTV		34
#define PREWITTH		35
#define LAPLACIANA1		36
#define LAPLACIANA2		37
#define LAPLACIANA3		38
#define LOG				39
#define SELECMAXIMO		40

#define	DOS_SEMIPERIODOS	0	// Tipo de Periodicidad en FFT
#define	PERIODO_COMPLETO	1

#define	REESCALADO		0	// Para las convoluciones
#define	MINMAX			1
#define VALOR_ABSOLUTO	2

#define SUAVE	1	// Para el efecto Emboss
#define MEDIO	3
#define FUERTE	5

#define MEDIAREGION  -1       // Para el Umbral de la Umbralizacin Local Adaptativa
#define MAXREGION    -2       
#define MAXIMAGEN    -3

#define	MINMAXREG		0	// Para los niveles de salida de la Umbralizacin Local Adaptativa 
#define	CEROMAXREG		1
#define MINMAXIMG		2
#define	RANGOTOTAL		3

#define CUADRADA		0		// Para el contraste
#define CUBICA			1
#define RAIZCUADRADA	2
#define RAIZCUBICA	    3
#define LOGARITMICA		4
#define SENOIDAL		5
#define TANGENCIAL		6

#define _1BPP	0		// Para cambiar el ColorDepth
#define _4BPP	1
#define _8BPP	2
#define _24BPP  3

#define	NUEVOTAM	0	// Para la traslacin
#define	TAMORIGINAL	1

#define	NTSC	0	// Para Color A Gris
#define	MEDIA	1

#define COMPLEMENTO   0     // Operaciones de conjuntos
#define REFLEXION     1
#define UNION         2
#define INTERSECCION  3
#define DIFERENCIAC   4

#define DILATACION   0     // Operaciones morfolgicas
#define EROSION      1
#define APERTURA     2
#define CIERRE       3

#define REDONDEO       0     // Para la ecualizacin del histograma (tipo de redondeo)
#define TRUNCAMIENTO   1
#define POR_EXCESO     2

#define DENTRO  0     // Para el efecto Hoover
#define FUERA   1

// Atributos
private:

    // Estructura del DIB
	BYTE *m_lpDIB;					// Puntero al comienzo del DIB
    LPBITMAPINFOHEADER m_lpBmIH;	// Puntero a estructura BITMAPINFOHEADER
	LPBITMAPINFO m_lpBmI;			// Puntero a estructura BITMAPINFO
	RGBQUAD *m_pPaleta;				// Paleta de colores 
	BYTE *m_pchMapaBits;			// Cuerpo de la Imagen 

	int m_iAnchoBitmapAlineado;		// N de columnas del bitmap (incluido el relleno
									// de alineacin). As, m_iAnchoDIB >= m_iColumnas
									// (1, 2  3 bytes mximo)
	int m_iNumNiveles;				// Niveles de intensidad de la imagen
	CString m_sNombre;				// Nombre de la imagen

// Operaciones
public:
	typedef struct InformacionHistograma
	{
		int NivelLuminanciaMin;	   // Nivel de intensidad con MinNPuntosLuminancia pxeles
		int NPuntosLuminanciaMin;  // N de pxeles con nivel MinNivelLuminancia
		int NivelRojoMin;
		int NPuntosRojoMin;
		int NivelVerdeMin;
		int NPuntosVerdeMin;
		int NivelAzulMin;
		int NPuntosAzulMin;

		int NivelLuminanciaMax;
		int NPuntosLuminanciaMax;
		int NivelRojoMax;
		int NPuntosRojoMax;
		int NivelVerdeMax;
		int NPuntosVerdeMax;
		int NivelAzulMax;
		int NPuntosAzulMax;

		int NivelMasLuminancia;
		int NPuntosMasLuminancia;
		int NivelMasRojo;
		int NPuntosMasRojo;
		int NivelMasVerde;
		int NPuntosMasVerde;
		int NivelMasAzul;
		int NPuntosMasAzul;
	} InfoHistogram;

	typedef struct CualesRGB
	{
		BOOL Luminancia;
		BOOL Rojo;
		BOOL Verde;
		BOOL Azul;
	} RGBWhich;

	typedef struct Punteros
	{
		DWORD DIB;
		DWORD BmIH;
		DWORD BmI;
		DWORD Paleta;
		DWORD MapaBits;
	} POINTERS;

	CImagen();						// Constructores y destructora
	CImagen(const BYTE *Bitmap, const RGBQUAD *Paleta, int Columnas, int Filas, int ColorDepth);
	virtual ~CImagen();
	CImagen& operator=(const CImagen&);	// Sobrecarga operador =

	//============================================================================
	int fnIMG_LoadBMP(CString);
	int fnIMG_LoadBMP(CFile *pFile);
	int fnIMG_LoadBMPFromBuffer(BYTE *Buffer, int size);
	int fnIMG_LoadRAW(CString);
	int fnIMG_LoadRAW(CString, int Columnas, int Filas);
	int fnIMG_LoadRAW(CFile *pFile);
	int fnIMG_LoadRAW(CFile *pFile, int Columnas, int Filas);
	int fnIMG_LoadRAWFromBuffer(BYTE *Buffer, int size);
	int fnIMG_LoadPGM(CString);
	int fnIMG_LoadPGM(CFile *pFile);
	int fnIMG_LoadPGMFromBuffer(BYTE *Buffer, int size);

	int fnIMG_SaveBMP(CString);
	int fnIMG_SaveBMP(CFile *pFile);
	int fnIMG_SaveRAW(CString);
	int fnIMG_SaveRAW(CFile *pFile);
	//============================================================================
	void fnIMG_GetBitmap(BYTE *);
	void fnIMG_SetBitmap(const BYTE *);
	int fnIMG_GetRows();
	int fnIMG_GetColumns();
	int fnIMG_GetBitmapSize();
	int fnIMG_GetLevels();
	int fnIMG_GetColorsUsed();
	int fnIMG_GetColorDepth();
	BITMAPINFOHEADER fnIMG_GetBitmapInfoHeader();
	POINTERS fnIMG_GetPointers();
	int fnIMG_GetDIBSize();
	void fnIMG_SetFileName(CString Nombre);
	CString fnIMG_GetFileName();
	CString fnIMG_GetFileExt();
	//=============================================================================
	BOOL fnIMG_IsInside(int Columna, int Fila);
	int fnIMG_Initialize(UINT Valor);
	BOOL fnIMG_IsGray();
	UINT fnIMG_GetPixel(int Columna, int Fila);
	int fnIMG_SetPixel(int Columna, int Fila, UINT Valor);
	int fnIMG_GetRGBPixel(int Columna, int Fila, RGBQUAD *Pal);
	int fnIMG_SetRGBPixel(int Columna, int Fila, RGBQUAD Pal);
	int fnIMG_GetPalette(RGBQUAD *Pal);
	int fnIMG_SetPalette(const RGBQUAD *Pal);
	int fnIMG_GetPaletteSize();
	int fnIMG_LoadPalette(CString PathArchivoEntrada);
	int fnIMG_SavePalette(CString PathArchivoSalida);
	HANDLE fnIMG_CopyToHandle();
	void fnIMG_CopyFromHandle(HANDLE hMem);
	int fnIMG_Draw(CDC *, int Columna, int Fila);
	int fnIMG_SplitRGB(CImagen *R, CImagen *G, CImagen *B);
	//=============================================================================
	int fnIMG_PseudoColor();
	int fnIMG_ChangeColorDepth(int Modo);
	int fnIMG_GetHistogram(int *Histograma, int Valor);
	int fnIMG_GetInfoHistogram(InfoHistogram *pIH);
	int fnIMG_EqualizeHistogram(int Modo);
	int fnIMG_GrayScaleLinealTransform(int Emin, int Emax, int Smin, int Smax, RGBWhich CRGB);
	int fnIMG_Contrast(int TipoFuncion, float Alfa);
	int fnIMG_Balance(int TantoPorCientoRojo, int TantoPorCientoVerde, int TantoPorCientoAzul);
	int fnIMG_Brightness(int TantoPorCiento);
	int fnIMG_Threshold(BYTE Umbral, BYTE NivelMin, BYTE NivelMax);
	int fnIMG_AdaptiveLocalThreshold(int AltoBloque, int AnchoBloque, int Umbral, int TipoUmbralizacion);
	int fnIMG_Negative();
	int fnIMG_Negative(RGBWhich CRGB);
	int fnIMG_ColorToGray(int Modo);
	int fnIMG_Rotation(int Modo, double Grados);
	int fnIMG_Scaling(double FactorX, double FactorY);
	int fnIMG_Translation(int Modo, int DesplColumnas, int DesplFilas);
	int fnIMG_Crop(int ColumnaSupIzq, int FilaSupIzq, int ColumnaInfDer, int FilaInfDer);
	int fnIMG_Ellipse();
	int fnIMG_Mosaic(int AltoBloque, int AnchoBloque);
	int fnIMG_Emboss(int Grado);
	int fnIMG_Hoover(int Modo, int Columna, int Fila, int NumIteraciones);
	int fnIMG_Flip(int Modo);
	int fnIMG_Arithmetic(int Operacion, CImagen *ImagenB);
	int fnIMG_Logic(int Operacion, CImagen *ImagenB);
	int fnIMG_SetOps(int Operacion, CImagen *ImagenB);
	int fnIMG_Morphological(int Operacion, int *Estructural, int DimX_EE, int DimY_EE, int CentroX_EE, int CentroY_EE);
	int fnIMG_FFT(int TipoPeriodicidad);
	int fnIMG_Bitplane(int Plano);
	int fnIMG_AddNoise(int TipoRuido, int Distribucion, int ValorMin, int ValorMax, int NivelMin, int NivelMax, int NumPuntos);
	int fnIMG_AverageRow(int Fila, RGBQUAD *Pal);
	int fnIMG_AverageColumn(int Columna, RGBQUAD *Pal);
	int fnIMG_AverageImg(RGBQUAD *Pal);
	int fnIMG_GetDifferentPixels(CImagen *ImagenB);
	int fnIMG_Filters(int TipoFiltro, int Modo);
	int fnIMG_Convolution(int *Mascara, int Dim, int FactorDiv, int Modo);
	
	//=============================================================================

protected:
	int fnIMG_GetAnchoBitmapAlineado();
	int fnIMG_GetAnchoBitmapAlineado(int NumColumnas, int biBitCount);
	int fnIMG_GetSizeBitmapAlineado();
	int fnIMG_GetSizeBitmapAlineado(int NumColumnas, int NumFilas, int bibitCount);
	BOOL fnIMG_TransfPixel(float x, float y, float &x1, float &y1); //Utilizada por EfectoElipse
	void fnIMG_Fourier(_complex f[], int ln);	// Implementacin de la FFT


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CImagen)
	//}}AFX_VIRTUAL

	// Generated message map functions
protected:
	//{{AFX_MSG(CImagen)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG

//	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_OBJETOIMAGEN_H__9E6BA040_AB96_11D2_9260_00C0DF46F28C__INCLUDED_)

/* ====================================================================================================

				DESCRIPCIN DE USO DE LA CLASE

//====================================================================================================

Los distintos datos y mtodos de la clase estn definidos en el archivo Imagen.h e 
implementados en los archivos  CImagen.dll y  CImagen.lib.
Para utilizar esta clase en cualquier proyecto hay que proceder del siguiente modo:


  Desde un proyecto en Visual C++ .NET 2003:

	1.- Incluir en el proyecto el archivo de definiciones de la clase (Imagen.h).
		Para ello:  Menu Proyecto/Agregar elemento existente

	2.- Situar el archivo CImagen.dll en el directorio /Windows/System o en el
		directorio por defecto del proyecto (opcin aconsejable para no ir llenando de
		basura el directorio Windows/System)

	3.- Situar el archivo CImagen.lib en el directorio /Microsoft Visual Studio .NET 2003/VC7/lib
		o en el directorio por defecto del proyecto (opcin recomendada).
		En este segundo caso hay que indicar al entorno de programacin donde se encuentra esa librera.
		Para ello:  Menu/Projecto/Propiedades de .../Vinculador y escribir en
		la opcin Entrada/Dependencias adicionales el nombre de la librera: CImagen.lib

  Desde un proyecto en Visual C++ 6.0:

	1.- Incluir en el proyecto el archivo de definiciones de la clase (Imagen.h)
		Menu Proyect/Add to Proyect/Files

	2.- Situar el archivo CImagen.dll en el directorio /Windows/System o en el
		directorio por defecto del proyecto (opcin aconsejable para no ir llenando de
		basura el directorio Windows/System)

	3.- Situar el archivo CImagen.lib en el directorio /Visual Studio/Vc98/Lib o en el 
		directorio por defecto del proyecto (opcin recomendada). En este segundo caso
		hay que indicar al entorno de programacin donde se encuentra esa librera.
		Para ello Menu/Project/Settings/Link y escribir en la opcin
		Object/Library modules el nombre de la librera: CImagen.lib


	Si se utiliza la arquitectura Documento/Vista de Visual C++ se puede operar de la siguiente
forma:

1.- En el archivo de cabecera del documento declarar un puntero a objeto de la
clase CImagen de la siguiente forma:

					CImagen *pImagen;

sin olvidarse de realizar el include a la cabecera de la clase:

#include "Imagen.h"

2.- En el constructor del archivo .cpp del documento reservar memoria para dicha instancia

                pImagen = new CImagen;

y en el destructor del documento:

		if(pImagen != NULL) delete pImagen; 

*/